#![deny(warnings)]

use std::env;
use std::fs::File;
use std::io;
use std::io::prelude::*;

use serde_json::Value as Json;
use milf::Value as Milf;

fn main() {
    let mut args = env::args();
    let mut input = String::new();
    if args.len() > 1 {
        let name = args.nth(1).unwrap();
        File::open(&name)
            .and_then(|mut f| f.read_to_string(&mut input))
            .unwrap();
    } else {
        io::stdin().read_to_string(&mut input).unwrap();
    }

    match input.parse() {
        Ok(milf) => {
            let json = convert(milf);
            println!("{}", serde_json::to_string_pretty(&json).unwrap());
        }
        Err(error) => println!("failed to parse MILF: {}", error),
    }
}

fn convert(milf: Milf) -> Json {
    match milf {
        Milf::String(s) => Json::String(s),
        Milf::Integer(i) => Json::Number(i.into()),
        Milf::Float(f) => {
            let n = serde_json::Number::from_f64(f).expect("float infinite and nan not allowed");
            Json::Number(n)
        }
        Milf::Boolean(b) => Json::Bool(b),
        Milf::Array(arr) => Json::Array(arr.into_iter().map(convert).collect()),
        Milf::Table(table) => {
            Json::Object(table.into_iter().map(|(k, v)| (k, convert(v))).collect())
        }
        Milf::Datetime(dt) => Json::String(dt.to_string()),
    }
}
